"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var ip = require("ip");
var url = require("url");
var requireModule_1 = require("./requireModule");
var pkg = requireModule_1.default('./package.json');
var spinConfig = pkg.spin;
var mobileAssetTest = /\.(bmp|gif|jpg|jpeg|png|psd|svg|webp|m4v|aac|aiff|caf|m4a|mp3|wav|html|pdf|ttf)$/;
var babelUsed = false;
var Platform = (function () {
    function Platform(preset) {
        this.features = preset.split('-');
        if (this.hasAny('server')) {
            this.target = 'server';
        }
        else if (this.hasAny('web')) {
            this.target = 'web';
        }
        else if (this.hasAny('android')) {
            this.target = 'android';
        }
        else if (this.hasAny('ios')) {
            this.target = 'ios';
        }
    }
    Platform.prototype.hasAny = function (arg) {
        var array = arg.constructor === Array ? arg : [arg];
        for (var _i = 0, array_1 = array; _i < array_1.length; _i++) {
            var feature = array_1[_i];
            if (this.features.indexOf(feature) >= 0) {
                return true;
            }
        }
        return false;
    };
    return Platform;
}());
exports.Platform = Platform;
var useBabel = function () {
    if (!babelUsed) {
        require('babel-register')({
            presets: ['es2015', 'flow'],
            ignore: /node_modules(?!\/(haul|react-native))/,
            retainLines: true,
            sourceMaps: 'inline',
        });
        require('babel-polyfill');
        babelUsed = true;
    }
};
var createBaseConfig = function (platform, dev, options) {
    var babelRule = {
        loader: 'babel-loader',
        options: {
            cacheDirectory: dev,
            presets: ["react", ["es2015", { "modules": false }], "stage-0"],
            plugins: [
                "transform-runtime",
                "transform-decorators-legacy",
                "transform-class-properties",
                ["styled-components", { "ssr": options.ssr }]
            ].concat(dev && options.reactHotLoader ? ['react-hot-loader/babel'] : []),
            only: ["*.js", "*.jsx"]
        }
    };
    var reactNativeRule = {
        loader: 'babel-loader',
        options: {
            cacheDirectory: dev,
            presets: ["react-native"],
            plugins: [
                path.join(process.cwd(), 'node_modules/haul/src/utils/fixRequireIssues')
            ]
        }
    };
    var baseConfig = {
        devtool: dev ? '#cheap-module-source-map' : '#source-map',
        module: {
            rules: [
                {
                    test: /\.jsx?$/,
                    exclude: platform.hasAny(['ios', 'android']) ?
                        /node_modules\/(?!react-native|@expo|expo|lottie-react-native|haul|pretty-format|react-navigation)$/ :
                        /node_modules/,
                    use: [
                        (platform.hasAny(['ios', 'android']) ?
                            function (req) {
                                var result;
                                if (req.resource.indexOf('node_modules') >= 0) {
                                    result = reactNativeRule;
                                }
                                else {
                                    result = babelRule;
                                }
                                return result;
                            } :
                            babelRule)
                    ].concat(options.persistGraphQL ?
                        ['persistgraphql-webpack-plugin/js-loader'] :
                        [])
                },
                {
                    test: /\.graphqls/,
                    use: 'raw-loader'
                },
                {
                    test: /\.(graphql|gql)$/,
                    exclude: /node_modules/,
                    use: ['graphql-tag/loader'].concat(options.persistGraphQL ?
                        ['persistgraphql-webpack-plugin/graphql-loader'] :
                        [])
                },
            ]
        },
        resolve: {
            extensions: platform.hasAny('server') ?
                [".web.js", ".web.jsx", '.js', '.jsx'] :
                ["." + platform.target + ".js", "." + platform.target + ".jsx", '.native.js', '.native.jsx', '.js', '.jsx']
        },
        watchOptions: {
            ignored: /build/
        },
        bail: !dev
    };
    if (platform.hasAny(['web', 'server'])) {
        baseConfig.resolve.alias = {
            'react-native': 'react-native-web'
        };
        baseConfig.module.rules = baseConfig.module.rules.concat([
            {
                test: /\.(png|ico|jpg|xml)$/,
                use: 'url-loader?name=[hash].[ext]&limit=10000'
            },
            {
                test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
                use: 'url-loader?name=./assets/[hash].[ext]&limit=10000'
            },
            {
                test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
                use: 'file-loader?name=./assets/[hash].[ext]'
            },
        ]);
    }
    else if (platform.hasAny(['android', 'ios'])) {
        baseConfig.module.rules = baseConfig.module.rules.concat([
            {
                test: mobileAssetTest,
                use: {
                    loader: require.resolve('./react-native/assetLoader'),
                    query: { platform: platform.target, root: path.resolve('.'), bundle: false },
                }
            }
        ]);
    }
    return baseConfig;
};
var persistPlugins;
var ExtractTextPlugin;
var createPlugins = function (platform, dev, options) {
    var webpack = requireModule_1.default('webpack');
    var buildNodeEnv = dev ? (platform.hasAny('test') ? 'test' : 'development') : 'production';
    if (!persistPlugins) {
        var PersistGraphQLPlugin = requireModule_1.default('persistgraphql-webpack-plugin');
        var moduleName = path.resolve('node_modules/persisted_queries.json');
        if (options.persistGraphQL) {
            var clientPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName,
                filename: 'extracted_queries.json', addTypename: true });
            var serverPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName,
                provider: clientPersistPlugin });
            persistPlugins = { client: clientPersistPlugin, server: serverPersistPlugin };
        }
        else {
            var clientPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName });
            var serverPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName });
            persistPlugins = { client: clientPersistPlugin, server: serverPersistPlugin };
        }
    }
    var plugins = [];
    if (dev) {
        plugins.push(new webpack.NamedModulesPlugin());
    }
    else {
        plugins.push(new webpack.optimize.UglifyJsPlugin({ minimize: true }));
        plugins.push(new webpack.LoaderOptionsPlugin({ minimize: true }));
        plugins.push(new webpack.optimize.ModuleConcatenationPlugin());
    }
    var backendUrl = options.backendUrl.replace('{ip}', ip.address());
    if (platform.hasAny('server')) {
        plugins = plugins.concat([
            new webpack.BannerPlugin({
                banner: 'require("source-map-support").install();',
                raw: true, entryOnly: false
            }),
            new webpack.DefinePlugin({
                __CLIENT__: false, __SERVER__: true, __SSR__: options.ssr,
                __DEV__: dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
                __PERSIST_GQL__: options.persistGraphQL,
                __BACKEND_URL__: "\"" + backendUrl + "\""
            }),
            persistPlugins.server
        ]);
    }
    else {
        plugins = plugins.concat([
            new webpack.DefinePlugin({
                __CLIENT__: true, __SERVER__: false, __SSR__: options.ssr,
                __DEV__: dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
                __PERSIST_GQL__: options.persistGraphQL,
                __BACKEND_URL__: (platform.target !== 'web' ||
                    url.parse(backendUrl).hostname !== 'localhost') ? "\"" + backendUrl + "\"" : false,
            }),
            persistPlugins.client
        ]);
        if (platform.hasAny('web')) {
            var ManifestPlugin = requireModule_1.default('webpack-manifest-plugin');
            plugins.push(new ManifestPlugin({
                fileName: 'assets.json'
            }));
            if (!spinConfig.presets['apollo-react-server']) {
                var HtmlWebpackPlugin = requireModule_1.default('html-webpack-plugin');
                plugins.push(new HtmlWebpackPlugin({
                    template: 'tools/html-plugin-template.ejs',
                    inject: 'body',
                }));
            }
            if (!dev) {
                ExtractTextPlugin = requireModule_1.default('extract-text-webpack-plugin');
                plugins.push(new ExtractTextPlugin({ filename: '[name].[contenthash].css', allChunks: true }));
                plugins.push(new webpack.optimize.CommonsChunkPlugin({
                    name: "vendor",
                    filename: "[name].[hash].js",
                    minChunks: function (module) {
                        return module.resource && module.resource.indexOf(path.resolve('./node_modules')) === 0;
                    }
                }));
            }
        }
        else if (platform.hasAny(['android', 'ios'])) {
            plugins.push(new webpack.SourceMapDevToolPlugin({
                test: /\.(js|jsx|css|bundle)($|\?)/i,
                filename: '[file].map',
            }));
        }
    }
    if (platform.hasAny('dll')) {
        var name = "vendor_" + platform.target;
        plugins = [
            new webpack.DefinePlugin({
                __DEV__: dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\""
            }),
            new webpack.DllPlugin({
                name: name,
                path: path.join(options.dllBuildDir, name + "_dll.json"),
            })
        ];
    }
    return plugins;
};
var getDepsForPlatform = function (platform, depPlatforms) {
    var deps = [];
    for (var _i = 0, _a = Object.keys(pkg.dependencies); _i < _a.length; _i++) {
        var key = _a[_i];
        var val = depPlatforms[key];
        if (!val || platform.hasAny(val)) {
            deps.push(key);
        }
    }
    if (platform.hasAny(['android', 'ios'])) {
        deps = deps.concat(require.resolve('./react-native/react-native-polyfill.js'));
    }
    return deps;
};
var createConfig = function (preset, dev, opts, depPlatforms) {
    var platform = new Platform(preset);
    var options = __assign({}, opts);
    if (platform.hasAny('test')) {
        options.ssr = false;
        options.persistGraphQL = false;
    }
    var baseDevServerConfig = {
        hot: true,
        contentBase: '/',
        publicPath: '/',
        headers: { 'Access-Control-Allow-Origin': '*' },
        quiet: false,
        noInfo: true,
        stats: { colors: true, chunkModules: false }
    };
    var config;
    if (platform.hasAny(['ios', 'android'])) {
        useBabel();
    }
    var plugins = createPlugins(platform, dev, options);
    if (platform.hasAny('server')) {
        var nodeExternals = requireModule_1.default('webpack-node-externals');
        var nodeExternalsFn_1 = nodeExternals({
            whitelist: [/(^webpack|^react-native)/]
        });
        config = __assign({}, createBaseConfig(platform, dev, options), { name: 'backend', entry: {
                index: [
                    'babel-polyfill',
                    './src/server/index.js'
                ]
            }, target: 'node', node: {
                __dirname: true,
                __filename: true
            }, externals: function (context, request, callback) {
                return nodeExternalsFn_1(context, request, function () {
                    if (request.indexOf('react-native') >= 0) {
                        return callback(null, 'commonjs ' + request + '-web');
                    }
                    else {
                        return callback.apply(this, arguments);
                    }
                });
            }, output: {
                devtoolModuleFilenameTemplate: dev ? '../../[resource-path]' : undefined,
                devtoolFallbackModuleFilenameTemplate: dev ? '../../[resource-path];[hash]' : undefined,
                filename: '[name].js',
                sourceMapFilename: '[name].[chunkhash].js.map',
                path: path.resolve(options.backendBuildDir),
                publicPath: '/'
            }, plugins: plugins });
        config.module.rules = config.module.rules.concat([{
                test: /\.scss$/,
                use: dev ? [
                    { loader: 'isomorphic-style-loader' },
                    { loader: 'css-loader', options: { sourceMap: true } },
                    { loader: 'postcss-loader', options: { sourceMap: true } },
                    { loader: 'sass-loader', options: { sourceMap: true } }
                ] :
                    [{ loader: 'ignore-loader' }]
            }]);
    }
    else if (platform.hasAny('web')) {
        var backendUrl = options.backendUrl.replace('{ip}', ip.address());
        var _a = url.parse(backendUrl), protocol = _a.protocol, host = _a.host;
        var backendBaseUrl = protocol + '//' + host;
        config = __assign({}, createBaseConfig(platform, dev, options), { name: 'web-frontend', entry: {
                index: [
                    'babel-polyfill',
                    './src/client/index.jsx'
                ]
            }, output: {
                filename: '[name].[hash].js',
                path: path.resolve(path.join(options.frontendBuildDir, 'web')),
                publicPath: '/'
            }, plugins: plugins, devServer: __assign({}, baseDevServerConfig, { port: options.webpackDevPort, proxy: {
                    '!/*.hot-update.{json,js}': {
                        target: backendBaseUrl,
                        logLevel: 'info'
                    }
                } }) });
        config.module.rules = config.module.rules.concat([{
                test: /\.scss$/,
                use: dev ? [
                    { loader: 'style-loader' },
                    { loader: 'css-loader', options: { sourceMap: true, importLoaders: 1 } },
                    { loader: 'postcss-loader', options: { sourceMap: true } },
                    { loader: 'sass-loader', options: { sourceMap: true } },
                ] : ExtractTextPlugin.extract({
                    fallback: "style-loader",
                    use: ['css-loader', 'postcss-loader', 'sass-loader']
                })
            }]);
    }
    else if (platform.hasAny(['android', 'ios'])) {
        var AssetResolver_1 = requireModule_1.default('haul/src/resolvers/AssetResolver');
        var HasteResolver = requireModule_1.default('haul/src/resolvers/HasteResolver');
        config = __assign({}, createBaseConfig(platform, dev, options), { name: platform.target + "-frontend", entry: {
                index: [
                    require.resolve('./react-native/react-native-polyfill.js'),
                    './src/mobile/index.js'
                ]
            }, output: {
                filename: "index.mobile.bundle",
                publicPath: '/',
                path: path.resolve(path.join(options.frontendBuildDir, platform.target)),
            }, devServer: __assign({}, baseDevServerConfig, { hot: false, port: platform.hasAny('android') ? 3010 : 3020 }), plugins: plugins });
        config.resolve.plugins = [
            new HasteResolver({
                directories: [path.resolve('node_modules/react-native')],
            }),
            new AssetResolver_1({ platform: platform.target, test: mobileAssetTest }),
        ];
        config.resolve.mainFields = ['react-native', 'browser', 'main'];
    }
    else {
        throw new Error("Unknown platform target: " + platform.target);
    }
    if (platform.hasAny('dll')) {
        var name = "vendor_" + platform.target;
        config = __assign({}, config, { devtool: '#cheap-module-source-map', name: platform.target + "-dll", entry: {
                vendor: getDepsForPlatform(platform, depPlatforms),
            }, output: {
                filename: name + ".[hash]_dll.js",
                path: path.resolve(options.dllBuildDir),
                library: name
            } });
    }
    return config;
};
exports.default = createConfig;
//# sourceMappingURL=generator.js.map